@@ -10,7 +10,9 @@
 #'
 #' @param excl Character string (or vector of character strings) denoting parameters to exclude. Used in conjunction with \code{params} argument to select parameters of interest.
 #'
-#' @param ISB Ignore Square Brackets (ISB). Logical specifying whether square brackets should be ignored in the \code{params} and \code{excl} arguments. If \code{TRUE}, square brackets are ignored - input from \code{params} and \code{excl} are otherwise matched exactly. If \code{FALSE}, square brackets are not ignored - input from \code{params} and \code{excl} are matched using grep, which can take arguments in regular expression format. This allows partial names to be used when specifying parameters of interest.
+#' @param ISB Ignore Square Brackets (ISB). Logical specifying whether square brackets should be ignored in the \code{params} and \code{excl} arguments. If \code{TRUE}, square brackets are ignored. If \code{FALSE}, square brackets are not ignored.  This allows partial names to be used when specifying parameters of interest. Use \code{exact} argument to specify whether input from \code{params} and \code{excl} arguments should be matched exactly.
+#'
+#' @param exact Logical specifying whether input from \code{params} and \code{excl} arguments should be matched exactly (after ignoring square brackets if \code{ISB = FALSE}). If \code{TRUE}, input from \code{params} and \code{excl} are matched exactly (after taking \code{ISB} argument into account). If \code{FALSE}, input from \code{params} and \code{excl} are matched using regular expression format (after taking \code{ISB} argument into account).
 #'
 #' @param probs Numeric vector where each element in (0,1) representing probabilities used to calculate posterior sample quantiles for the selected parameters. Default is c(0.025, 0.5, 0.975).
 #' 
@@ -31,13 +33,13 @@
 #' @param func_name Character string (or vector of character strings) specifying labels for output from \code{func} argument. If \code{func_name} is not specified, columns with \code{func} argument will be labeled 'func'.
 #'
 #' @section Details:
-#' \code{object} argument can be a \code{stanfit} object (\code{rstan} package), a \code{stanreg} object (\code{rstanarm} package), a \code{brmsfit} object (\code{brms} package), an \code{mcmc.list} object (\code{coda} package), an \code{R2jags} model object (\code{R2jags} package), a \code{jagsUI} model object (\code{jagsUI} package), or a matrix containing MCMC chains (each column representing MCMC output for a single parameter, rows representing iterations in the chain). The function automatically detects the object type and proceeds accordingly.
+#' \code{object} argument can be a \code{stanfit} object (\code{rstan} package), a \code{stanreg} object (\code{rstanarm} package), a \code{brmsfit} object (\code{brms} package), an \code{mcmc.list} object (\code{coda} and \code{rjags} packages), \code{mcmc} object (\code{coda} and \code{nimble} packages), \code{list} object (\code{nimble} package), an \code{R2jags} model object (\code{R2jags} package), a \code{jagsUI} model object (\code{jagsUI} package), or a matrix containing MCMC chains (each column representing MCMC output for a single parameter, rows representing iterations in the chain). The function automatically detects the object type and proceeds accordingly.
 #'
 #' @section Notes:
 #'
-#' For \code{mcmc.list} objects, the potential scale reduction statistic statistic (Rhat) is calculated using the \code{gelman.diag} function in the \code{coda} package (what is typically displayed in the summary output from models fit with JAGS). For \code{stanfit} (as well as \code{stanreg} and \code{brmsfit} objects) and \code{jagsUI} objects, Rhat is calculated using a 'split chain' Rhat (in their respective packages), which is thought to be a more conservative diagnostic (Stan Development Team 2018).
+#' For \code{mcmc.list}, \code{mcmc}, and \code{list} objects, the potential scale reduction statistic statistic (Rhat) is calculated using the \code{gelman.diag} function in the \code{coda} package (what is typically displayed in the summary output from models fit with JAGS). For \code{stanfit} (as well as \code{stanreg} and \code{brmsfit} objects) and \code{jagsUI} objects, Rhat is calculated using a 'split chain' Rhat (in their respective packages), which is thought to be a more conservative diagnostic (Stan Development Team 2018).
 #'
-#' For \code{mcmc.list} objects, the number of effective samples is calculated using the \code{effectiveSize} function in the \code{coda} package. For \code{stanfit} (as well as \code{stanreg} and \code{brmsfit} objects) and \code{jagsUI} objects, n.eff is calculated using a slightly different method of computation for the number of effective samples (Stan Development Team 2018).
+#' For \code{mcmc.list}, \code{mcmc}, and \code{list} objects, the number of effective samples is calculated using the \code{effectiveSize} function in the \code{coda} package. For \code{stanfit} (as well as \code{stanreg} and \code{brmsfit} objects) and \code{jagsUI} objects, n.eff is calculated using a slightly different method of computation for the number of effective samples (Stan Development Team 2018).
 #'
 #' @return Function returns summary information (including parameter posterior mean, posterior sd, quantiles, potential scale reduction statistic (Rhat), number of effective samples, and other specified metrics) for specified parameters.
 #'
@@ -46,7 +48,7 @@
 #'
 #' Brooks, S. P., and A. Gelman. 1998. General methods for monitoring convergence of iterative simulations. Journal of Computational and Graphical Statistics 7:434.
 #'
-#' Stan Development Team. 2018. Stan Modeling Language Users Guide and Reference Manual, Version 2.18.0.   http://mc-stan.org
+#' Stan Development Team. 2018. Stan Modeling Language Users Guide and Reference Manual, Version 2.18.0.   https://mc-stan.org
 #'
 #' @examples
 #' #Load data
@@ -59,47 +61,52 @@
 #' MCMCsummary(MCMC_data, params = 'beta', round = 2)
 #'
 #' #Just 'beta[1]', 'beta[4]', and 'alpha[3]'
-#' #'params' takes regular expressions when ISB = FALSE, square brackets must be escaped with '\\'
-#' MCMCsummary(MCMC_data, params = c('beta\\[1\\]', 'beta\\[4\\]', 'alpha\\[3\\]'), ISB = FALSE)
+#' MCMCsummary(MCMC_data, params = c('beta[1]', 'beta[4]', 'alpha[3]'), ISB = FALSE, exact = TRUE)
 #'
 #' @export
 
 MCMCsummary <- function(object,
-                      params = 'all',
-                      excl = NULL,
-                      ISB = TRUE,
-                      probs = c(0.025, 0.5, 0.975),
-                      hpd_prob = 0.95,
-                      HPD = FALSE,
-                      digits = NULL,
-                      round = NULL,
-                      Rhat = TRUE,
-                      n.eff = TRUE,
-                      func = NULL,
-                      func_name = NULL) {
-  
-#--------------------------------------------------------------------------------------------------------------                        
-# SORTING BLOCK
-                      
-  if (methods::is(object, 'matrix')) {
-    object2 <- MCMCchains(object, params, excl, ISB, mcmc.list = FALSE)
+                        params = 'all',
+                        excl = NULL,
+                        ISB = TRUE,
+                        exact = TRUE,
+                        probs = c(0.025, 0.5, 0.975),
+                        hpd_prob = 0.95,
+                        HPD = FALSE,
+                        digits = NULL,
+                        round = NULL,
+                        Rhat = TRUE,
+                        n.eff = TRUE,
+                        func = NULL,
+                        func_name = NULL)
+{ 
+  # SORTING BLOCK
+
+  if (methods::is(object, 'matrix'))
+  {
+    object2 <- MCMCchains(object, params, excl, ISB, exact = exact, mcmc.list = FALSE)
   } else {
-    if (methods::is(object, 'stanfit')) {
+    if (methods::is(object, 'stanfit'))
+    {
       object2 <- object
     } else {
       # rstanarm
-      if (methods::is(object, 'stanreg')) {
+      if (methods::is(object, 'stanreg'))
+      {
         object2 <- object$stanfit
       } else {
         # brms
-        if (methods::is(object, 'brmsfit')) {
+        if (methods::is(object, 'brmsfit'))
+        {
           object2 <- object$fit
         } else {
           #jagsUI
-          if (methods::is(object, 'jagsUI')) { 
+          if (methods::is(object, 'jagsUI'))
+          { 
             object2 <- MCMCchains(object)
+          } else {
+            object2 <- MCMCchains(object, params, excl, ISB, exact = exact, mcmc.list = TRUE)
           }
-          object2 <- MCMCchains(object, params, excl, ISB, mcmc.list = TRUE)
         }
       }
     }
@@ -194,9 +201,9 @@ MCMCsummary <- function(object,
       colnames(bind_mn) <- "mean"  
       colnames(bind_sd) <- "sd"  
       
-      if (HPD==FALSE)
+      if (HPD == FALSE)
       {
-        if (length(probs)==1)
+        if (length(probs) == 1)
         {
           bind_q <- data.frame(apply(ch_bind, 2, stats::quantile, probs = probs))
           colnames(bind_q) <-  paste0(signif(probs * 100, digits =3), "%")
@@ -346,7 +353,7 @@ MCMCsummary <- function(object,
       rs_df <- data.frame(object$summary)
     }
     
-    # filtering of parameters from rstan object - from MCMCchains
+    # filtering of parameters from rstan/jagsUI object - from MCMCchains
     if (ISB == TRUE)
     {
       names <- vapply(strsplit(all_params, split = "[", fixed = TRUE), `[`, 1, FUN.VALUE = character(1))
@@ -365,22 +372,26 @@ MCMCsummary <- function(object,
       {
         if (ISB == TRUE)
         {
-          n_excl <- vapply(strsplit(excl, split = "[", fixed = TRUE), `[`, 1, FUN.VALUE = character(1))
-          ind_excl <- which(names %in% n_excl[i])
-          if (length(ind_excl) < 1)
-          {
-            warning(paste0("\"", excl[i], "\"", " not found in MCMC output."))
-          }
-          rm_ind <- c(rm_ind, ind_excl)
+          n_excl <- vapply(strsplit(excl,
+                                    split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
         } else {
           n_excl <- excl
-          ind_excl <- grep(n_excl[i], names, fixed = FALSE)
-          if (length(ind_excl) < 1) {
-            warning(paste0("\"", excl[i], "\"", " not found in MCMC output."))
-          }
-          rm_ind <- c(rm_ind, ind_excl)
         }
+        
+        if (exact == TRUE)
+        {
+          ind_excl <- which(names %in% n_excl[i])
+        } else {
+          ind_excl <- grep(n_excl[i], names, fixed = FALSE)
+        }
+        
+        if (length(ind_excl) < 1)
+        {
+          warning(paste0("\"", excl[i], "\"", " not found in MCMC output. Check 'ISB'' and 'exact' arguments to make sure the desired parsing methods are being used."))
+        }
+        rm_ind <- c(rm_ind, ind_excl)
       }
+      
       if (length(rm_ind) > 0)
       {
         dups <- which(duplicated(rm_ind))
@@ -408,7 +419,7 @@ MCMCsummary <- function(object,
           f_ind <- (1:length(names))[-rm_ind2]
         }
       } else {
-        if (ISB == TRUE)
+        if (exact == TRUE)
         {
           get_ind <- which(names %in% params)
         } else {
@@ -417,7 +428,7 @@ MCMCsummary <- function(object,
         
         if (length(get_ind) < 1)
         {
-          stop(paste0("\"", params, "\"", " not found in MCMC output."))
+          stop(paste0("\"", params, "\"", " not found in MCMC output. Check 'ISB' and 'exact' arguments to make sure the desired parsing methods are being used."))
         }
         if (!is.null(excl))
         {
@@ -440,7 +451,7 @@ MCMCsummary <- function(object,
       grouped <- c()
       for (i in 1:length(params))
       {
-        if (ISB == TRUE)
+        if (exact == TRUE)
         {
           get_ind <- which(names %in% params[i])
         } else {
@@ -449,7 +460,7 @@ MCMCsummary <- function(object,
         
         if (length(get_ind) < 1)
         {
-          warning(paste0("\"", params[i], "\"", " not found in MCMC output."))
+          warning(paste0("\"", params[i], "\"", " not found in MCMC output. Check 'ISB' and 'exact' arguments to make sure the desired parsing methods are being used."))
           (next)()
         }
         grouped <- c(grouped, get_ind)
@@ -488,8 +499,8 @@ MCMCsummary <- function(object,
 # end sort
 
 # convert object to matrix if computing non default intervals or using custom func
-    if (!is.null(func) | HPD==TRUE | 
-        identical(probs, c(0.025, 0.5, 0.975))==FALSE)
+    if (!is.null(func) | HPD == TRUE | 
+        identical(probs, c(0.025, 0.5, 0.975)) == FALSE)
     {
       if (methods::is(object2, 'stanfit'))
       {
@@ -516,14 +527,14 @@ MCMCsummary <- function(object,
       colnames(bind_mn) <- "mean"  
       colnames(bind_sd) <- "sd"  
       
-      if (HPD==FALSE)
+      if (HPD == FALSE)
       {
         if (length(probs)==1)
         {
           bind_q <- data.frame(signif(apply(ch_bind, 2, stats::quantile, probs = probs), digits = digits))
           colnames(bind_q) <-  paste0(signif(probs * 100, digits = 3), "%")
         } else {
-          if (identical(probs, c(0.025, 0.5, 0.975))==TRUE)
+          if (identical(probs, c(0.025, 0.5, 0.975)) == TRUE)
           {
             bind_LCI <- signif(rs_df["X2.5."][f_ind, 1], digits = digits)
             bind_med <- signif(rs_df["X50."][f_ind, 1], digits = digits)
@@ -540,7 +551,7 @@ MCMCsummary <- function(object,
       {
         if (length(hpd_prob) > 1)
         {
-          stop('specify only a single probability for HPD interval computation.')
+          stop('Specify only a single probability for HPD interval computation.')
         }
         bind_q <- data.frame(signif(coda::HPDinterval(coda::as.mcmc(ch_bind), prob = hpd_prob), digits = digits))
         colnames(bind_q) <- c(paste0(signif(hpd_prob * 100, digits = 3), "%_HPDL"), paste0(signif(hpd_prob * 100, digits = 3), "%_HPDU"))  
@@ -554,14 +565,14 @@ MCMCsummary <- function(object,
       colnames(bind_mn) <- "mean"  
       colnames(bind_sd) <- "sd"  
       
-      if (HPD==FALSE)
+      if (HPD == FALSE)
       {
         if (length(probs)==1)
         {    
           bind_q <- data.frame(round(apply(ch_bind, 2, stats::quantile, probs = probs), digits = round))
           colnames(bind_q) <-  paste0(signif(probs * 100, digits = 3), "%")         
         } else {
-          if (identical(probs, c(0.025, 0.5, 0.975))==TRUE)
+          if (identical(probs, c(0.025, 0.5, 0.975)) == TRUE)
           {    
             bind_LCI <- round(rs_df["X2.5."][f_ind, 1], digits = round)
             bind_med <- round(rs_df["X50."][f_ind, 1], digits = round)
@@ -578,7 +589,7 @@ MCMCsummary <- function(object,
       {
         if (length(hpd_prob) > 1)
         {
-          stop('specify only a single probability for HPD interval computation.')
+          stop('Specify only a single probability for HPD interval computation.')
         }
         bind_q <- data.frame(round(coda::HPDinterval(coda::as.mcmc(ch_bind), prob = hpd_prob), digits = round))
         colnames(bind_q) <- c(paste0(signif(hpd_prob * 100, digits = 3), "%_HPDL"), paste0(signif(hpd_prob * 100, digits = 3), "%_HPDU"))  
@@ -592,14 +603,14 @@ MCMCsummary <- function(object,
       colnames(bind_mn) <- "mean"  
       colnames(bind_sd) <- "sd"  
       
-      if (HPD==FALSE)
+      if (HPD == FALSE)
       {
         if (length(probs)==1)
         {
           bind_q <- data.frame(apply(ch_bind, 2, stats::quantile, probs = probs))
           colnames(bind_q) <-  paste0(signif(probs * 100, digits = 3), "%")                    
         } else {
-          if (identical(probs, c(0.025, 0.5, 0.975))==TRUE)
+          if (identical(probs, c(0.025, 0.5, 0.975)) == TRUE)
           {    
             bind_LCI <- rs_df["X2.5."][f_ind, 1]
             bind_med <- rs_df["X50."][f_ind, 1]
@@ -616,7 +627,7 @@ MCMCsummary <- function(object,
       {
         if (length(hpd_prob) > 1)
         {
-          stop('specify only a single probability for HPD interval computation.')
+          stop('Specify only a single probability for HPD interval computation.')
         }
         bind_q <- data.frame(coda::HPDinterval(coda::as.mcmc(ch_bind), prob = hpd_prob))
         colnames(bind_q) <- c(paste0(signif(hpd_prob * 100, digits = 3), "%_HPDL"), paste0(signif(hpd_prob * 100, digits = 3), "%_HPDU"))  
